@@ -3,6 +3,7 @@
 # Discus is a program that reports hard drive space usage.
 
 # Copyright 2000 Stormy Henderson (stormy@futuresouth.com).
+# Copyright 2019-2021 Nicolas Carrier (carrier.nicolas0@gmail.com).
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,18 +20,74 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307
 # USA.
 import subprocess
-import getopt
 import os
 import sys
 import re
-import unittest
 import copy
 import shutil
 from collections import namedtuple
+import argparse
+from argparse import RawTextHelpFormatter
 
-opts = {"placing": True, "reserved": True}
 
-VERSION = "0.4.0"
+# These colors should work on VT100-type displays and can be overridden by the
+# user.
+black = "\033[30m"
+red = "\033[31m"
+green = "\033[32m"
+yellow = "\033[33m"
+blue = "\033[34m"
+magenta = "\033[35m"
+cyan = "\033[36m"
+white = "\033[37m"
+
+on_black = "\033[40m"
+on_red = "\033[41m"
+on_green = "\033[42m"
+on_yellow = "\033[43m"
+on_blue = "\033[44m"
+on_magenta = "\033[45m"
+on_cyan = "\033[46m"
+on_white = "\033[47m"
+
+normal = "\033[0m"
+bold = "\033[1m"
+underline = "\033[4m"
+blink = "\033[5m"
+reverse = "\033[7m"
+dim = "\033[8m"
+
+opts = {
+    "placing": True,
+    "reserved": True,
+    # Labels to display next to numbers.
+    "akabytes": ["KB", "MB", "GB", "TB", "PB", "EB"],
+    "color": 1,
+    # Power of 1024, smallest value is 0, for KB, used only when smart
+    # formatting is disabled, overridden by -t, -g, -m and -k options
+    "divisor": 1,
+    "graph": 1,
+    "graph_char": "*",
+    "graph_fill": "-",
+    # Example mtab entry that uses a shell command (always use a ! as
+    # first character) rather than a file:
+    #        !/bin/mount |awk '{print $1, $3}'"
+    "mtab": "/etc/mtab",
+    #  Number of decimal places to display, same as -p
+    "places": 1,
+    # Filesystems to ignore.
+    "skip_list": ["/dev/pts", "/proc", "/dev", "/proc/usb", "/sys"],
+    #  Use smart formatting of numbers.
+    "smart": 1,
+    "color_header": blue,
+    "color_normal": normal,
+    "color_safe": normal,             # 0%- 70% full
+    "color_warning": bold + yellow,   # 70%- 85% full
+    "color_danger": bold + red        # 85%-100% full
+}
+
+HOMEPAGE = "https://github.com/ncarrier/discus"
+VERSION = "0.5.0"
 MINIMUM_WIDTH = 68
 
 # values taken from sysexit.h
@@ -66,36 +123,6 @@ class StatsFactory:
         return StatsFactory.Stats(total=total, free=free, used=used)
 
 
-class StatsFactoryTests(unittest.TestCase):
-    """Unit tests for the StatsFactory class"""
-    def test_getStatsReservedTrue(self):
-        """Normal use case with reserved == True."""
-        factory = StatsFactory(True)
-        s = factory.getStats("/")
-        self.assertNotEqual(s.total, 0, "a size of 0 for / is unlikely")
-        self.assertNotEqual(s.free, 0, "0 bytes free for / is unlikely")
-        self.assertNotEqual(s.used, 0, "0 bytes used for / is unlikely")
-
-    def test_getStatsReservedFalse(self):
-        """Normal use case with reserved == False."""
-        factory = StatsFactory(False)
-        s = factory.getStats("/")
-        self.assertNotEqual(s.total, 0, "a size of 0 for / is unlikely")
-        self.assertNotEqual(s.free, 0, "0 bytes free for / is unlikely")
-        self.assertNotEqual(s.used, 0, "0 bytes used for / is unlikely")
-        self.assertEqual(s.total, s.free + s.used, "total != free + used")
-
-    def test_getStatsFailure(self):
-        """Failure use case, non existent mount point."""
-        factory = StatsFactory(False)
-        raised = False
-        try:
-            factory.getStats("non existent mount point")
-        except Exception:
-            raised = True
-        self.assertTrue(raised)
-
-
 class SizeFormatter:
     """
     Class responsible of formatting sizes, smartly or not.
@@ -106,8 +133,8 @@ class SizeFormatter:
     DEFAULT_AKABYTES = ["KB", "MB", "GB", "TB", "PB", "EB"]
     # helper class for manipulating options
     Options = namedtuple("Options", ["smart", "placing", "akabytes", "places",
-                                     "divisor"],
-                         defaults=(True, True, DEFAULT_AKABYTES, 1, 1))
+                                     "divisor"])
+    Options.__new__.__defaults__ = (True, True, DEFAULT_AKABYTES, 1, 1)
 
     def __init__(self, smart, placing, akabytes, places, divisor):
         """Constructor, initialize private fields."""
@@ -173,44 +200,6 @@ class SizeFormatter:
         return size, divisor, self.__places
 
 
-class SizeFormatterTests(unittest.TestCase):
-    """Unit tests for the SizeFormatter class"""
-
-    def test_manual_format(self):
-        """Format sizes in manual format mode."""
-        # kilo bytes
-        opts = SizeFormatter.Options(smart=False, divisor=0)
-        sf = SizeFormatter(*opts)
-        # TODO, this is not correct, if the converted size is exact, then there
-        # should be no .0 part, but to implement that, we have to manipulate
-        # sizes internally in bytes, not in kilo bytes
-        self.assertEqual(sf.format(124684), "124684.0 KB", "124684 in KB fail")
-        # mega bytes
-        opts = SizeFormatter.Options(smart=False, divisor=1)
-        sf = SizeFormatter(*opts)
-        self.assertEqual(sf.format(124684), "121.8 MB", "124684 in MB fail")
-        # giga bytes
-        opts = SizeFormatter.Options(smart=False, divisor=2)
-        sf = SizeFormatter(*opts)
-        self.assertEqual(sf.format(124684), "0.1 GB", "124684 in GB fail")
-        # tera bytes
-        opts = SizeFormatter.Options(smart=False, divisor=3)
-        sf = SizeFormatter(*opts)
-        self.assertEqual(sf.format(pow(1024, 3)), "1.0 TB", "1 TB in TB fail")
-        self.assertEqual(sf.format(0), "0 TB", "0 TB in TB fail")
-
-    def test_smart_format(self):
-        """Format sizes in smart format mode."""
-        opts = SizeFormatter.Options(smart=True)
-        sf = SizeFormatter(*opts)
-        self.assertEqual(sf.format(124684), "121.8 MB", "124684 fail")
-        self.assertEqual(sf.format(1024), "1.0 MB", "1024 fail")
-        self.assertEqual(sf.format(1), "1.0 KB", "1 fail")
-        self.assertEqual(sf.format(0), "0 KB", "0 fail")
-        self.assertEqual(sf.format(999), "999.0 KB", "999 fail")
-        self.assertEqual(sf.format(1000), "1.0 MB", "1000 fail")
-
-
 Mount = namedtuple('Mount', ['mount', 'device'])
 
 
@@ -242,25 +231,6 @@ class DiskData:
                              mount.device)
 
 
-class DiskDataTests(unittest.TestCase):
-    """Unit tests for the DiskData class"""
-
-    def test_get(self):
-        """Tests for the get method."""
-        sf = SizeFormatter(*SizeFormatter.Options())
-        mount = Mount("/", "/dev/sda1")
-        d = DiskData.get(StatsFactory.Stats(1000000, 1000000 - 50000, 50000),
-                         5.0,
-                         mount,
-                         sf)
-        self.assertEqual(d.percent, "5.0%", "percent doesn't match")
-        self.assertEqual(d.total, "976.6 KB", "total doesn't match")
-        self.assertEqual(d.used, "48.8 KB", "used doesn't match")
-        self.assertEqual(d.free, "927.7 KB", "free doesn't match")
-        self.assertEqual(d.mount, mount.mount, "mount doesn't match")
-        self.assertEqual(d.device, mount.device, "device doesn't match")
-
-
 class Disk:
     """Contains everything needed to represent a disk textually."""
 
@@ -289,7 +259,7 @@ class Disk:
         """Format a percentage as a bar graph."""
         # How many stars to place?
         # -4 accounts for the [] and the starting space
-        width = width - 3
+        width = width - 4
         if isinstance(percent, str):
             bar_width = 0
         else:
@@ -299,10 +269,7 @@ class Disk:
         result = color("safe")
         graph_char = opts["graph_char"]
         graph_fill = opts["graph_fill"]
-        for counter in range(1, width + 1):
-            if counter > bar_width:
-                break
-
+        for counter in range(0, bar_width):
             if counter >= 0.7 * width and counter < 0.85 * width:
                 result = result + color("warning")
             elif counter >= 0.85 * width:
@@ -310,7 +277,7 @@ class Disk:
 
             result = result + graph_char
 
-        result = result + (width - counter) * graph_fill
+        result = result + (width - bar_width) * graph_fill
 
         return "  [" + color("safe") + result + color("normal") + "]"
 
@@ -333,104 +300,50 @@ class Disk:
         return text
 
 
-def version():
-    """Print version."""
-    print("Discus version {} by Nicolas Carrier "
-          "(carrier.nicolas0@gmail.com).".format(VERSION))
-    print("Home page: https://github.com/ncarrier/discus")
-    sys.exit(0)
+def parse_options(args=sys.argv[1:]):
+    """"""
+    parser = argparse.ArgumentParser(description=f"Discus version {VERSION}, "
+                                     "to display disk usage.",
+                                     formatter_class=RawTextHelpFormatter)
+    parser.add_argument("-d", "--device", action="store_true",
+                        default=False, help="show device instead of graph")
+    parser.add_argument("-c", "--color", action="store_false", default=True,
+                        help="disable color")
+    parser.add_argument("-g", "--gigabytes", action="store_const", const=2,
+                        dest="divisor", help="display sizes in gigabytes")
+    parser.add_argument("-k", "--kilobytes", action="store_const", const=0,
+                        dest="divisor", help="display sizes in kilobytes")
+    parser.add_argument("-m", "--megabytes", action="store_const", const=1,
+                        dest="divisor", help="display sizes in megabytes")
+    parser.add_argument("-p", "--places", type=int, choices=range(0, 10),
+                        help="number of digits to right of decimal place")
+    parser.add_argument("-r", "--reserved", action="store_true",
+                        default=False, help="count reserved space as used")
+    parser.add_argument("-s", "--smart", action="store_false", default=True,
+                        help="do not use smart formatting")
+    parser.add_argument("-t", "--terabytes", action="store_const", const=3,
+                        dest="divisor", help="display sizes in terabytes")
+    parser.add_argument("-v", "--version", action="version",
+                        version=(f"Discus version {VERSION} by Nicolas "
+                                 "Carrier (carrier.nicolas0@gmail.com)\n"
+                                 f"Home page: {HOMEPAGE}"))
+
+    return parser.parse_args(args)
 
 
-def usage(exit_status, text=""):
-    """Print the help of the tool."""
-    # Only print the general help if no specific message is provided.
-    if text == "":
-        print("""Discus version %s, to display disk usage.
-Options are:
- -h, --help      display help
- -v, --version   display version
- -p number       number of digits to right of decimal place
- -s              do not use smart formatting
- -t, g, m, k     display sizes in tera/giga/mega/kilobytes, repectively.
-                    assumes -s
- -d              show device instead of graph
- -c              disable color
- -r              count reserved space as used
-""" % VERSION)
-
-    else:
-        print(text)
-
-    sys.exit(exit_status)
-
-
-def parse_options():
-    """Read the user's options and integrate them with the defaults."""
-    try:
-        options, _ = getopt.getopt(sys.argv[1:], "p:tgmksdcrvh",
-                                   ["help", "version"])
-    except Exception:
-        sys.exit(EX_USAGE)
-
-    for option, value in options:
-        # Display terabytes.
-        if option == "-t":
-            opts["divisor"] = 3
-            opts["smart"] = 0
-            opts["places"] = 3
-
-        # Display gigabytes.
-        if option == "-g":
-            opts["divisor"] = 2
-            opts["smart"] = 0
-            opts["places"] = 2
-
-        # Display megabytes.
-        if option == "-m":
-            opts["divisor"] = 1
-            opts["smart"] = 0
-            opts["places"] = 1
-
-        # Display kilobytes.
-        if option == "-k":
-            opts["divisor"] = 0
-            opts["smart"] = 0
-            opts["places"] = 0
-
-        # Display X decimal places.
-        if option == "-p":
-            opts["placing"] = True
-            fail = False
-            try:
-                opts["places"] = int(value)
-            except ValueError:
-                fail = True
-
-            if fail or opts["places"] < 0 or opts["places"] > 9:
-                usage(EX_USAGE, "The -p option requires a number from 0 to 9.")
-
-        # Disable smart display.
-        if option == "-s":
-            opts["smart"] = 0
-
-        # Show device instead of graph.
-        if option == "-d":
-            opts["graph"] = 0
-
-        # Turn off color.
-        if option == "-c":
-            opts["color"] = 0
-
-        # Display version.
-        if option in ["-v", "--version"]:
-            version()
-
-        # Display help.
-        if option in ["-h", "--help"]:
-            usage(EX_OK)
-
-        if option == "-r":
-            opts["reserved"] = 1
+def interpret_options(o):
+    opts["smart"] = 1 if o.smart else 0
+    if o.divisor is not None:
+        if o.places is None:
+            opts["places"] = o.divisor
+        opts["divisor"] = o.divisor
+        opts["smart"] = 0
+    if o.places is not None:
+        opts["placing"] = True
+        opts["places"] = o.places
+    opts["graph"] = 0 if o.device else 1
+    opts["color"] = 1 if o.color else 0
+    opts["reserved"] = 1 if o.reserved else 0
 
 
 def read_mounts(mtab, skip_list):
@@ -468,22 +381,6 @@ def read_mounts(mtab, skip_list):
     return mounts
 
 
-class ReadMountsTests(unittest.TestCase):
-    """Tests for the read_mounts function."""
-    def test_simple_mtab(self):
-        """Test with a simple mtabe consisting of only one line."""
-        mounts = read_mounts("tests/mtab.oneline", [])
-        self.assertEqual(len(mounts), 1, "one device should be detected")
-        self.assertEqual(mounts[0].device, "/dev/sda2")
-        self.assertEqual(mounts[0].mount, "/")
-
-    def test_bug_291276(self):
-        """Test to reproduce the debian bug 291276."""
-        mounts = read_mounts("tests/mtab.291276", [])
-        self.assertEqual(len(mounts), 1, "one mount point should be detected")
-        self.assertEqual(mounts[0].mount, "/media/ACER UFD")
-
-
 def color(code):
     """Function that return color codes if color is enabled."""
     if opts["color"]:
@@ -513,13 +410,13 @@ def format_fields(f, w):
 def get_layout(headers, reports):
     graph_column_width = 14
     widths = [11, 11, 12, 12, 8, graph_column_width]
-    inputs = [copy.deepcopy(headers)] + copy.deepcopy(reports)
+    data = [copy.deepcopy(headers)] + copy.deepcopy(reports)
 
     size = shutil.get_terminal_size((MINIMUM_WIDTH, 20))
     # limit the width to a minimum and account to the inter-column gap
     columns = max(MINIMUM_WIDTH, size.columns - len(widths))
-    for l in inputs:
-        for i, v in enumerate(l[:-1]):
+    for datum in data:
+        for i, v in enumerate(datum[:-1]):
             if len(v) > widths[i]:
                 widths[i] = len(v)
 
@@ -533,7 +430,8 @@ def get_layout(headers, reports):
 
 def main():
     """Define main program."""
-    parse_options()
+    options = parse_options()
+    interpret_options(options)
     mounts = read_mounts(opts["mtab"], opts["skip_list"])
     headers = get_header(opts["graph"])
     stats_factory = StatsFactory(opts["reserved"])
@@ -558,14 +456,14 @@ def main():
 
 
 if __name__ == "__main__":
-    # Before starting, we need to load the configuration files which
-    # contain global objects.  First the global /etc file, then the user's
-    # file, if exists.
+    # Before starting, we try to load the configuration files which may
+    # override global objects' values.
+    # First the global /etc file, then the user's file, if it exists.
     try:
         exec(compile(open("/etc/discusrc", "rb").read(), "/etc/discusrc",
              'exec'))
     except IOError:
-        usage(EX_CONFIG, "/etc/discusrc must exist and be readable.")
+        pass
 
     try:
         exec(compile(open(os.environ['HOME'] + "/.discusrc", "rb").read(),
